;*=====================================================================*/
;*    Author      :  Manuel Serrano, Florian Loitsch                   */
;*    Copyright   :  2009 Manuel Serran, Florian Loitsch, see LICENSE file */
;*    -------------------------------------------------------------    */
;*    This file is part of Scheme2Js.                                  */
;*                                                                     */
;*   Scheme2Js is distributed in the hope that it will be useful,      */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   LICENSE file for more details.                                    */
;*=====================================================================*/

;; dsssl handling.
;; copied (originally verbatim) from bigloo.

(module runtime-dsssl
   (export (dsssl-get-key-arg (JS "sc_dsssl_get_key_arg")
			      (arity 3)
			      (constant? #t))
	   (dsssl-get-key-rest-arg (JS "sc_dsssl_get_key_rest_arg")
				   (arity 1)
				   (constant? #t))
	   (dsssl-check-key-args! (JS "sc_dsssl_check_key_args")
				  (arity 2)
				  (constant? #t))))

;*---------------------------------------------------------------------*/
;*    dsssl-check-key-args! ...                                        */
;*    -------------------------------------------------------------    */
;*    This function checks that dsssl args are, at runtime,            */
;*    correctly formed. That is, the dsssl-args variable must hold     */
;*    a serie of pairs where the first element is a keyword.           */
;*    Furthermore, if key-list is non-nil, we check that for each      */
;*    pair, if the key is present in key-list.                         */
;*---------------------------------------------------------------------*/
(define (dsssl-check-key-args! dsssl-args key-list)
   (if (null? key-list)
       (let loop ((args dsssl-args))
	  (cond
	     ((null? args)
	      dsssl-args)
	     ((or (not (pair? args))
		  (null? (cdr args))
		  (not (keyword? (car args))))
	      (error "dsssl formal parsing"
		     "Unexpected #!keys parameters"
		     args))
	     (else
	      (loop (cddr args)))))
       (let loop ((args dsssl-args)
		  (armed #f)
		  (opts '()))
	  (cond
	     ((null? args)
	      (reverse! opts))
	     ((or (not (pair? args))
		  (null? (cdr args))
		  (not (keyword? (car args)))
		  (not (memq (car args) key-list)))
	      (if (not armed)
		  (loop (cdr args) armed opts)
		  (loop (cdr args)
			#f
			(cons (car args) opts))))
	     (else
	      (loop (cddr args) #t opts))))))
   
;*---------------------------------------------------------------------*/
;*    dsssl-get-key-arg ...                                            */
;*    -------------------------------------------------------------    */
;*    dsssl args have already been tested. We know for sure that       */
;*    it is a serie of pairs where first elements are keywords.        */
;*---------------------------------------------------------------------*/
(define (dsssl-get-key-arg dsssl-args keyword initializer)
   (let loop ((args dsssl-args))
      (cond
	 ((null? args)
	  initializer)
	 ((not (keyword? (car args)))
	  (loop (cdr args)))
	 ((eq? (car args) keyword)
	  (if (not (pair? (cdr args)))
	      (error 'dsssl-get-key-arg
		     "keyword argument misses value"
		     (car args))
	      (cadr args)))
	 (else
	  (if (not (pair? (cdr args)))
	      (error 'dsssl-get-key-arg
		     "keyword argument misses value"
		     (car args))
	      (loop (cddr args)))))))
   
;*---------------------------------------------------------------------*/
;*    dsssl-get-key-rest-arg ...                                       */
;*---------------------------------------------------------------------*/
(define (dsssl-get-key-rest-arg dsssl-args)
   (let loop ((args dsssl-args))
      (cond
	 ((null? args)
	  '())
	 ((or (not (keyword? (car args))) (null? (cdr args)))
	  args)
	 (else
	  (loop (cddr args))))))
